@@ -6,7 +6,7 @@ from time import sleep
 import socketio
 from aiortc import RTCPeerConnection, RTCRtpSender, RTCConfiguration, RTCIceServer, RTCSessionDescription
 from aiortc.contrib.media import MediaPlayer, MediaRelay
-
+from datetime import datetime
 try:
     import picamera
 except:
@@ -16,6 +16,7 @@ except:
 from gspeerconnection.pitrack import PiH264Relay, PiH264CameraOutput
 
 video_trackList = list()
+indexmap = {}
 pcs = set()
 relay = None
 microfon = None
@@ -39,6 +40,7 @@ class GSPeerConnectionBroadcasterPiCaller:
         self.sio = socketio.AsyncClient()
         self.gsdbs = gsdbs
         self.camera = camera
+        self._logger = logging.getLogger(__name__)
         self.oncommand = oncommand
 
     async def startSocket(self):
@@ -51,10 +53,11 @@ class GSPeerConnectionBroadcasterPiCaller:
 
 class GSPeerConnectionBroadcasterPiSocket:
 
-    def getPiH264Relay(self):
+    def getPiH264Relay(self, id):
         video_track = PiH264Relay(30)
 
         video_trackList.append(video_track)
+        indexmap[id] = video_trackList.index(video_track)
         return video_track
 
     def getMicrofonRelay(self):
@@ -72,7 +75,7 @@ class GSPeerConnectionBroadcasterPiSocket:
         self = GSPeerConnectionBroadcasterPiSocket()
 
         self.rtconfiList = []
-
+        self._logger = logging.getLogger(__name__)
         self.gsdbs = gsdbs
         self.oncommand = oncommand
         # picam.append(camera)
@@ -87,7 +90,7 @@ class GSPeerConnectionBroadcasterPiSocket:
 
         camera.resolution = (self.gsdbs.credentials["hres"], self.gsdbs.credentials["vres"])
         camera.framerate = self.gsdbs.credentials["framerate"]
-        sleep(1)  # camera warm-up time
+        sleep(1)
         target_bitrate = camera.resolution[0] * \
                          camera.resolution[1] * \
                          camera.framerate * 0.150
@@ -106,7 +109,7 @@ class GSPeerConnectionBroadcasterPiSocket:
 
         broadcast_thread.start()
         self.peerConnections = {}
-        self._logger = logging.getLogger(__name__)
+
         self.webcam = None
         self.relay = None
 
@@ -119,6 +122,7 @@ class GSPeerConnectionBroadcasterPiSocket:
             self._logger.debug("watcher found")
             if type(description) == str:
                 description = ast.literal_eval(description)
+
             desc = type('new_dict', (object,), description)
 
             offer = RTCSessionDescription(sdp=description["sdp"], type=description["type"])
@@ -132,16 +136,28 @@ class GSPeerConnectionBroadcasterPiSocket:
 
             self.peerConnections[id] = pc
 
-            video = self.getPiH264Relay()
+            video = self.getPiH264Relay(id)
             audio = None
 
-            @pc.on("connectionstatechange")
-            async def on_connectionstatechange():
-                print("Connection state is %s" % pc.connectionState)
-                if pc.connectionState == "failed":
-                    self._logger.info("closeConnection")
-                    await pc.close()
-                    self.peerConnections.pop(id, None)
+            def channel_log(channel, t, message):
+                self._logger.debug("channel(%s) %s %s" % (channel.label, t, message))
+
+            def channel_send(channel, message):
+                channel_log(channel, ">", message)
+                channel.send(message)
+
+            @pc.on("datachannel")
+            def on_datachannel(channel):
+
+                channel_log(channel, "-", "created by remote party")
+
+                @channel.on("message")
+                def on_message(message):
+                    channel_log(channel, "<", message)
+
+                    if isinstance(message, str) and message.startswith("ping"):
+                        channel_send(channel, f'{{"ping":"pong","ts":"{datetime.utcnow()}"}}' )
+
 
             transceiver = pc.addTransceiver("video", direction="sendonly")
 
@@ -182,9 +198,9 @@ class GSPeerConnectionBroadcasterPiSocket:
             if id in self.peerConnections:
                 self._logger.info("Disconnected Peer")
                 await self.peerConnections[id].close()
+                video_trackList.pop(indexmap[id])
                 self.peerConnections.pop(id, None)
 
-
         @self.sio.event
         async def disconnect():
             self._logger.info('disconnected from server')
